%Belyk, September 2017
%The basic idea of this script is to load predefined processing jobs built
%in the GUI with one sample scan in mind, and it automate replacing some of
%its settings (mostly filepaths) to process the remaining scans in the same
%way. This script depends on being able to load the structurestic (realign,
%coregister, smooth*, segment, and normalize. *smooth is not implemented
%here.
%helpful modifactions contributed by Elise Kanber, 2021

%data should be in 4D.nii format and organized such that each participant
%has a folder within which each scan has a folder.
%each scan folder contains a subfolder called deisgn. This contains one
%.csv file per condition in the run. Each file is a one row vector
%identifying the start time of a condition (in volumes).


%{
EK notes: 
This script was run for an experiment that had an event-related design with
three runs. To ensure that the design matrix contained all three runs in
the same spm.mat file, I looped through each run and wrote the data to a
single struct (ffx). 
As the ffx template only included one run (i.e. one row of data in the struct fields), 
it was necessary to specify all of the parameters for the remaining runs (even those that 
were to be left as defaults or blank), to ensure
each field was filled. 


%} 
clear variables

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%STUFF THAT NEEDS SETTING%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%list of subject folder names
subjects = {'CL250621'};

%list of run names, should also be name of run folders
runs = {'run1', 'run2','run3'};
no_runs = length(runs); %count 'em


% This is useful if you have your onsets etc. saved to a .txt file
% %list of conditions
% conds = {"low", "comfort", "mid", "high"};
% no_conds = length(conds); %count 'em

%runs to skip, missing data
skip_runs=[""];

% If you have a block design:
%number of volumes per block
% block_dur = 8;
% 
% %the number of blocks per condition
% no_blocks = 4;

%in seconds
TR = 2.45;

% units for design - do you want to give spm your timings in scans or
% seconds. Set to either 'scans' or 'secs.' 
units = 'secs';

%where the data is
basepath = '/Users/carolynmcgettigan/Documents/Exp2/';


% %for testing only - if you want to test just one condition from one subj
% from one run. 
% s = subjects{1};
% r = runs{2};
% c = conds{1};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%Fixed Effects Analysis%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load("ffx.mat")

tic %a time thingy
for s = subjects %loop through subjects
   
    %steps to do once per subject go here
    
    disp('%%%%%%%%%%%%%%%%%%%%%%%%') 
    disp(strcat("Start Subject: ", s))
    disp('%%%%%%%%%%%%%%%%%%%%%%%%') 
    
    w = 1; % We will use this to write each run to the same struct, rather than separate structs
    
    for r = runs %loop through runs in subjects
        %skip missing runs
        if ismember(strcat(s,r), skip_runs)%should this run be skipped
            disp(strcat("%%% Skipping:", s," ",r,"","%%%"))
        else      
            
            
            %build path to BOLD folders
            boldpath = fullfile(basepath, s, '/functional/', r);
            
            %where condition onsets are stored
            designpath = strcat(basepath, s, '/log/');
            
            
            % Condition names, onsets, and durations were saved in a .mat
            % file. 
            % Load in .mat onsets file
            load(strcat(designpath, s, "_", "onsets_", r, ".mat"));
            
            % List of conditions: 
            conds = names;  % my condition names were saved as 'names' in the .mat onsets file.
            no_conds = length(conds); %count 'em
           
            % load conditions onsets 
            ffx_onsets = cueonsets;
            
            % Load durations: 
            ffx_durations = durations;
            
            %number of volumes per functional run - useful if you have
            %different numbers of volumes in each run.
            hdr = spm_vol(strcat(boldpath, '/', 'sw', r, '.nii'));
            n_volumes = length(hdr{1});
                 
            
            % If onsets saved in a txt/csv file: load in conditions onset times
%             ffx_onsets = zeros(no_blocks,no_conds); %empty matrix for filling
%             col = 0; %row looping through rows of empty matrix
%             for c = conds
%                 col = col +1; %iterate
%                 new_onsets = csvread(strcat(designpath,r,"_",c{1},".csv"));%read in onset times for each condition
%                 ffx_onsets(:,col) = new_onsets; %store conditions in matrix. rows = blocks, cols = conditions
%             end
        
            %setup ffx job
            ffx{1}.spm.stats.fmri_spec.dir = boldpath; %where you want to save the SPM.mat file to 
            ffx{1}.spm.stats.fmri_spec.timing.RT = TR; %time per volume (repetition time)
            ffx{1}.spm.stats.fmri_spec.timing.units = units; % do you want to give spm your data in scans/seconds
            
            % Matlab writes the scans horizontally by default (i.e. new scan in each
            % column), but we need it to write vertically. This line of code
            % is useful for filling empty cell arrays vertically, which can
            % then be overwritten by the actual data. 
            ffx{1}.spm.stats.fmri_spec.sess(w).scans = cell(n_volumes, 1);
            
            for v = 1:n_volumes %loop through volumes to point to  the data, sw signifies coregistered, normalised, and smoothed data
                
                % where are the volumes
                ffx{1}.spm.stats.fmri_spec.sess(w).scans(v) = strcat(boldpath,'/sw',r,'.nii,',num2str(v));
                % multiple conditions - wasn't necessary for my design, so
                % filled with 0x0 character array 
                ffx{1}.spm.stats.fmri_spec.sess(w).multi  = cellstr(''); 
                % Regressors can be specified here
                ffx{1}.spm.stats.fmri_spec.sess(w).regress = struct([]);
                % Multiple regressors here - I selected movement parameters
                % file (rp_...txt)
                ffx{1}.spm.stats.fmri_spec.sess(w).multi_reg = strcat(boldpath, '/rp_', r, '.txt');
                % High pass filter - 128 is the default.
                ffx{1}.spm.stats.fmri_spec.sess(w).hpf = 128;
            end
            
            
            %{
            Loops through the different conditions and specifies the condition names,
            onsets, and durations.
            Have also specified the tmod, pmod, and orth columns (can be
            found by clicking through the ffx struct
            ffx{1}.spm.stats.fmri_spec.sess.cond(c).name) as I have more
            conditions than MB's example & these fields needed to be
            filled. 
            %}
            
            for c = 1:size(conds,2)%loop through conditions, numerically this time
                ffx{1}.spm.stats.fmri_spec.sess(w).cond(c).name = conds{c}; % condition names
                ffx{1}.spm.stats.fmri_spec.sess(w).cond(c).onset = ffx_onsets{c}; %block/event onset times
                ffx{1}.spm.stats.fmri_spec.sess(w).cond(c).duration = durations{c}; %block/event durations - can change
                                                                                 % this to zero if u want
                
                % Specified these using the spm defaults (usually won't need to change these):                                                               
                ffx{1}.spm.stats.fmri_spec.sess(w).cond(c).tmod = 0;
                ffx{1}.spm.stats.fmri_spec.sess(w).cond(c).pmod = struct([]);
                ffx{1}.spm.stats.fmri_spec.sess(w).cond(c).orth = 1;
                   
                
            end
            w = w+1;  % w here refers to the row in the struct to be filled. This will correspond to the run number
        end%finish else
            
        
       %execute ffx
        spm_jobman('run', ffx)
        disp(strcat("Done running ffx: ", s,'/', r)) 
        
 
    end %finish run loop
    toc %time for subject
end %finish subject loop
      
       % Still need to Estimate the spm.mat file that is created either
       % using the GUI and clicking Estimate, or by writing some code here.
            
            disp('%%%') 
            disp('Fin') %message to console
            disp('%%%')       